<?php
/**
 * Plugin Name: Simple Product Sample for WooCommerce
 * Plugin URI: https://mci-desarrollo.es/plugins
 * Author: MCI Desarrollo
 * Version: 1.2.9
 * Author URI: https://mci-desarrollo.es
 * Text Domain: simple-product-sample
 * Description: Add a button to "Request sample" on the WooCommerce product page. It is possible to activate the sample and configure its price for each product.
 **/
//=======================================================================
// Exit if accessed directly
if (!defined('ABSPATH')) {
    exit;
}

/////////////////// Defines current versión number as constant

if (!defined('MCISPS_VERSION')) {
    define('MCISPS_VERSION', '1.2.9');
}
/////////////////////////////////////////////////////////////////////////

// Load the textdomain
load_plugin_textdomain('simple-product-sample', false, basename(dirname(__FILE__)) . '/languages');

// Link to the admin style sheet
function mcisps_enqueue_admin_style()
{
    wp_enqueue_style('mcisps_admin', plugins_url('css/mcisps_admin_style.css', __FILE__));
}
add_action('admin_enqueue_scripts', 'mcisps_enqueue_admin_style');

// Link to the front style sheet
function mcisps_enqueue_front_style()
{
    wp_enqueue_style('mcisps_front', plugins_url('css/mcisps_front_style.css', __FILE__));
}
add_action('wp_enqueue_scripts', 'mcisps_enqueue_front_style');

////////////////////////////////////////////////////////////////////////////

require_once 'includes/mcisps_activation.php';

require_once 'includes/mcisps_unnistall.php';

require_once 'includes/mcisps_form_custom_fields.php';

require_once 'models/mcisps_set_custom_fields.php';

require_once 'includes/mcisps_panel.php';

require_once 'models/mcisps_options.php';

////////////////////////////////////////////////////////////////////////////////////////////////////

//Get max_samples_quantity if exists. If not exists set a default value
if (get_mcisps_option_from_db('max_samples') !== null) {
    $max_samples_quantity = get_mcisps_option_from_db('max_samples');
} else {
    $max_samples_quantity = '3';
}

////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////// CREATE BUTTON "Request a sample" AND ADD TO CART ///////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////

function mcisps_add_button()
{
    //Get max_samples_quantity if exists. If not exists set a default value
    if (get_mcisps_option_from_db('max_samples') !== null) {
        $max_samples_quantity = get_mcisps_option_from_db('max_samples');
    } else {
        $max_samples_quantity = '3';
    }

    global $product;
    $id_product = $product->get_id($product);

    global $woocommerce;

    if (get_post_meta($id_product, '_sample_active', true) == 'yes') {

        if (isset($_POST['add_sample'])) {

            if ($product->is_type('variable')) {

                //Get $available_variations in this product
                $available_variations = $product->get_available_variations(); //get all child variations
                $variation_id         = $available_variations[0]['variation_id'];
                $variations_terms     = $product->get_variation_attributes(); // get all attributes by variations

                $quantity                   = sanitize_text_field($_POST['quantity']);
                $passed_validation_variable = apply_filters('woocommerce_add_to_cart_validation', true, $variation_id, $quantity);

                if ($passed_validation_variable == 'sample_true') {
                    $woocommerce->cart->add_to_cart();
                    //The notice is shown automatically when adding the variable product / sample
                } else {
                    wc_add_notice(sprintf('You have reached the maximum of %s samples.', sanitize_text_field($max_samples_quantity), 'simple-product-sample'), 'error');
                }

            } else {

                $quantity                 = sanitize_text_field($_POST['quantity']);
                $passed_validation_simple = apply_filters('woocommerce_add_to_cart_validation', true, $id_product, $quantity);

                if ($passed_validation_simple == 'sample_true') {

                    //The product is added in mcisps_add_product_and_notice_in_single

                } else {
                    wc_add_notice(sprintf('You have reached the maximum of %s samples.', sanitize_text_field($max_samples_quantity), 'simple-product-sample'), 'error');
                }
            }

        } // end submit button $_POST['add_sample']

        $quantity              = 1;
        $passed_validation_btn = apply_filters('woocommerce_add_to_cart_validation', true, $id_product, $quantity);

        // Check if validation add_to_cart is passed and add button
        if (($product->is_type('variable') && $passed_validation_btn == 'sample_true') || ($product->is_type('simple') && $passed_validation_btn == 'sample_true')) {

            $variable_product_classes = 'class="sps_btn button"';
            echo '<button type="submit" name="add_sample" id="add_sample" ' . $variable_product_classes . '>' . __("Request a sample", "simple-product-sample") . '</button>';

        } else {
            $variable_product_classes = 'class="sps_btn sps_max button"';
            $title_max_message        = ' title="' . __('Maximum number of samples reached') . '"';
            echo '<div ' . $variable_product_classes . $title_max_message . '>' . sprintf("%s Samples max.", sanitize_text_field($max_samples_quantity), "simple-product-sample") . '</div>';
        }

    } //end if sample_active

} // end function
add_action('woocommerce_after_add_to_cart_button', 'mcisps_add_button', 10);

///////////// Add product & Display sample added to cart message for simple products /////////////////////////////////
function mcisps_add_product_and_notice_in_single()
{
    global $woocommerce;
    global $product;
    $id_product = $product->get_id($product);

    if (isset($_POST['add_sample'])) {
        if ($product->is_type('simple')) {

            $quantity                 = 1;
            $passed_validation_simple = apply_filters('woocommerce_add_to_cart_validation', true, $id_product, $quantity);

            if ($passed_validation_simple == 'sample_true') {

                $woocommerce->cart->add_to_cart($id_product);

                //echo '<script type="text/javascript">window.location.href="' . get_permalink() . '"</script>'; //reload page manually

                wc_print_notice(sprintf('Sample has been added to cart. <a href="' . wc_get_cart_url() . '" class="mcisps_show_cart">' . __('Show Cart', 'simple-product-sample') . '</a>', 'simple-product-sample')); //The notice is added manually
            }
        }
    }
}
add_action('woocommerce_before_single_product_summary', 'mcisps_add_product_and_notice_in_single', 10);

///////////// Edit sample added to cart message for variable products /////////////////////////////////
function mcisps_add_to_cart_message_html($message)
{
    $message = sprintf("Sample has been added to cart.", "simple-product-sample");
    return $message;
}
if (isset($_POST['add_sample'])) {
    add_filter('wc_add_to_cart_message_html', 'mcisps_add_to_cart_message_html', 10, 1);
}

///////// CONVERT THE VARIABLE PRODUCT IN SIMPLE IF THE BOX IS CHECKED ///////////////////////////////////

function mcisps_woocommerce_add_to_cart_handler($product_type, $product)
{
    global $wpdb;
    $tabla_mcisps_options = sanitize_text_field($wpdb->prefix) . 'mcisps_options';

    $get_option = $wpdb->get_results("SELECT mcisps_value FROM $tabla_mcisps_options WHERE mcisps_option = 'no_variations'");
    $get_option = $get_option[0]->mcisps_value;

    if ($get_option == 1) {
        $product_type = 'simple';
    }

    return $product_type;
}
if (isset($_POST['add_sample'])) {
    add_filter("woocommerce_add_to_cart_handler", "mcisps_woocommerce_add_to_cart_handler", 10, 2);
}

//////// VALIDATE FOR ADD TO CART /////////////////////////////////////////////////////////////////////////

function mcisps_add_to_cart_validation($passed, $product_id, $quantity)
{
    //Get max_samples_quantity if exists. If not exists set a default value
    if (get_mcisps_option_from_db('max_samples') !== null) {
        $max_samples_quantity = get_mcisps_option_from_db('max_samples');
    } else {
        $max_samples_quantity = '3';
    }

    global $woocommerce;

// Check number of samples are in the Cart
    $cart_items = $woocommerce->cart->get_cart();

    $samples_sum = 0;
    foreach ($cart_items as $item_value) {
        if (isset($item_value['_sample_active'])) {
            if ($item_value['_sample_active'] == 'yes') {
                if (isset($samples_sum)) {
                    $samples_sum += $item_value['quantity'];
                } else {
                    $samples_sum = $item_value['quantity'];
                }
            }
        }
    }

    $res_cart_contents = apply_filters('woocommerce_cart_contents', 1);

    if ($samples_sum < $max_samples_quantity && $quantity == 1 && $res_cart_contents <= $max_samples_quantity) {
        $passed = 'sample_true';
        $passed = true;
    } else {
        $passed = 'sample_false';
    }
    return $passed;
}
add_filter('woocommerce_add_to_cart_validation', 'mcisps_add_to_cart_validation', 15, 3);

////////////////////////////// VALIDATE WHEN UPDATE CART /////////////////////////////////////////////
///////////////////// COUNT SAMPLES IN CART //////////////////////////////////////////////////////////
function mcisps_count_samples_in_cart()
{
    global $woocommerce;

    $cart_items = $woocommerce->cart->get_cart();

    $samples_sum = 0;
    foreach ($cart_items as $item_value) {
        if (isset($item_value['_sample_active'])) {
            if ($item_value['_sample_active'] == 'yes') {
                if (isset($samples_sum)) {
                    $samples_sum += $item_value['quantity'];
                } else {
                    $samples_sum = $item_value['quantity'];
                }
            }
        }
    }

    return $samples_sum;

}
add_filter('woocommerce_cart_contents', 'mcisps_count_samples_in_cart', 15, 0);

//////////// DISPLAY ERROR MESSAGE CART AND BLOCK ORDER IF IS GREATER THAN MAX //////////////////////////
function mcisps_check_cart_quantities()
{
    //Get max_samples_quantity if exists. If not exists set a default value
    if (get_mcisps_option_from_db('max_samples') !== null) {
        $max_samples_quantity = get_mcisps_option_from_db('max_samples');
    } else {
        $max_samples_quantity = '3';
    }

    $passed_validation_cart = apply_filters('woocommerce_cart_contents', 0, 0); // Value from return $samples_sum

    if ($passed_validation_cart > $max_samples_quantity) {
        wc_add_notice(sprintf('Add a maximum of %s samples to Cart', $max_samples_quantity, 'simple-product-sample'), 'error');
    }
}
add_action('woocommerce_check_cart_items', 'mcisps_check_cart_quantities');

/////////////////////// MODIFY PRODUCT SAMPLE DATA IN CART ///////////////////////////////////////////////
///////////////////// ADD CUSTOM DATA FIELDS TO CART ITEM ////////////////////////////////////////////////
function mcisps_add_cart_item_data($cart_item_data, $product_id, $variation_id)
{

    $quantity          = 1;
    $passed_validation = apply_filters('woocommerce_add_to_cart_validation', true, $product_id, $quantity);

    if (isset($_POST['add_sample']) && $passed_validation == true) {
        $cart_item_data['_sample_price']  = get_post_meta($product_id, '_sample_price', true);
        $cart_item_data['_sample_active'] = get_post_meta($product_id, '_sample_active', true);
    }
    return $cart_item_data;
}
add_filter('woocommerce_add_cart_item_data', 'mcisps_add_cart_item_data', 10, 3);

///////////////////// REPLACE PRICE AND PRODUCT NAME IN CART ///////////////////////////////////////////////
function mcisps_before_calculate_totals($cart_item)
{
    if (is_admin() && !defined('DOING_AJAX')) {
        return;
    }

    // Iterate through each cart item
    $samples_sum = 0;

    foreach ($cart_item->get_cart() as $key => $value) {

        if (isset($value['_sample_price']) && ($value['_sample_active'] == 'yes')) {
            if ($value['_sample_price'] != '') {
                $price = $value['_sample_price'];
            } else {
                $price = 0;
            }
            $value['data']->set_price($price);

            $sample_name = __('SAMPLE', 'simple-product-sample');
            $sample_name .= ': ' . $value['data']->get_name();
            $value['data']->set_name($sample_name);

        }
    }

}
add_action('woocommerce_before_calculate_totals', 'mcisps_before_calculate_totals', 10, 1);